from django import forms
from django.utils.translation import gettext_lazy as _

FORM_CHOICES = (("all", _("All fields")), ("title", _("Title")), ("upc", _("UPC")))


class ProductSearchForm(forms.Form):
    q = forms.CharField(max_length=255, required=False, label=_("Search"))
    context = forms.ChoiceField(
        choices=FORM_CHOICES, initial="all", widget=forms.RadioSelect
    )
    field_order = ["q", "upc", "title"]



from django.utils.translation import gettext_lazy as _

from ocyan.core.fender import config


def price_ranges(default):
    ranges = config.get("oscar_elasticsearch", "price_ranges")
    if ranges is not None:
        return parse_range_definition(ranges)

    return default


def parse_range_definition(range_definition):
    return [
        int(range_str) for range_str in range_definition.split(",") if range_str != ""
    ]


DEFAULT_FACETS = [
    {
        "name": "price",
        "label": _("Price"),
        "type": "range",
        "formatter": "oscar_elasticsearch.search.format.currency",
        "ranges": price_ranges(default=[25, 100, 500, 1000]),
    }
]


def get_facet_config():
    return DEFAULT_FACETS + [
        dict(
            facet,
            name="attrs.%(name)s" % facet,
            type=facet["type"],
            label=facet["label"],
            ranges=parse_range_definition(facet.get("ranges", "")),
        )
        for facet in config.get("oscar_elasticsearch", "facets", [])
    ]



# -*- coding: utf-8 -*-
"""
Expose exactly one plugin config class at import time so straight.plugin can
discover it without loading both variants (which can cause duplicate app labels).
"""

from ocyan.plugin.oscar.info import OSCAR_NEEDS_OXYAN

if OSCAR_NEEDS_OXYAN:
    from .entrypoint import Oscar3ElasticsearchConfig as OscarElasticsearchConfig
else:
    from .entrypoint import OscarElasticsearchConfig

__all__ = ["OscarElasticsearchConfig"]



# pylint: skip-file
from django.dispatch import receiver

from wagtail.contrib.search_promotions.models import Query

from oscar_elasticsearch.search.signals import query_hit


@receiver(query_hit)
def handle_query_hit(sender, querystring, **kwargs):  # pylint: disable=unused-argument
    query_recorder = Query.get(querystring)
    query_recorder.add_hit()



import logging

from django.core.management import call_command
from django.db import connection
from uwsgidecoratorsfallback import spool

from oscar.core.loading import get_classes

logger = logging.getLogger(__name__)
(
    update_index_product,
    _update_index_products,
    update_index_category,
    _update_index_categories,
) = get_classes(
    "search.helpers",
    [
        "update_index_product",
        "update_index_products",
        "update_index_category",
        "update_index_categories",
    ],
)


@spool(pass_arguments=True)
def update_index_products(*product_ids):
    # the spooler is forked so the db connection might be closed
    try:
        if not connection.is_usable():
            connection.close()
    except AttributeError:
        connection.close_if_unusable_or_obsolete()

    _update_index_products(product_ids)


@spool(pass_arguments=True)
def update_index_categories(*category_ids):
    # the spooler is forked so the db connection might be closed
    try:
        if not connection.is_usable():
            connection.close()
    except AttributeError:
        connection.close_if_unusable_or_obsolete()

    _update_index_categories(category_ids)


@spool(pass_arguments=True)
def update_index(**kwargs):
    # the spooler is forked so the db connection might be closed
    try:
        if not connection.is_usable():
            connection.close()
    except AttributeError:
        connection.close_if_unusable_or_obsolete()

    category_id = kwargs.get("category_id")
    product_id = kwargs.get("product_id")

    if category_id is not None:
        update_index_category(category_id)

    if product_id is not None:
        update_index_product(product_id)


# pylint: disable=unused-argument
@spool
def update_oscar_index(arguments):
    # the spooler is forked so the db connection might be closed
    try:
        if not connection.is_usable():
            connection.close()
    except AttributeError:
        connection.close_if_unusable_or_obsolete()

    try:
        call_command("update_oscar_index")
    except Exception as e:  # pylint: disable=broad-except
        logger.exception(e)



# -*- coding: utf-8 -*-
from django.utils.text import slugify

from ocyan.core.fender import config
from ocyan.core.utils import merge_dicts
from ocyan.plugin.oscar.plugins import OscarPluginConfig

from .utils import get_facet_config

DEFAULT_ITEMS_PER_PAGE = config.get("oscar", "dashboard_items_per_page", 21)
PROJECT_NAME = config.get("django", "name")


class OscarElasticsearchConfig(OscarPluginConfig):
    def contribute_to_core_apps(self, core_apps):
        return core_apps + ["ocyan.plugin.oscar_elasticsearch.search"]

    def contribute_to_installed_apps(self, installed_apps):
        installed_apps.oscar.append(
            "oscar_elasticsearch.search", replaces="oscar.apps.search"
        )
        installed_apps.oscar.append(
            "ocyan.plugin.oscar_elasticsearch.search",
            replaces="oscar_elasticsearch.search",
        )
        installed_apps.oscar.append(
            "ocyan.plugin.oscar_elasticsearch.catalogue_es_extension",
            extends="ocyan.plugin.oscar_catalogue.catalogue",
        )
        installed_apps.oscar.register_extension(
            "ocyan.plugin.oscar_elasticsearch.catalogue_dashboard_es_extension.dashboard.catalogue",
            "ocyan.plugin.oscar_catalogue_dashboard.dashboard.catalogue",
        )
        installed_apps.wagtail.append("wagtail.contrib.search_promotions")
        return installed_apps

    def contribute_to_settings(self, settings):
        OSCAR_ELASTICSEARCH_OVERWRITE_SETTINGS = {
            "OSCAR_ELASTICSEARCH_QUERY_PAGE_SIZE": config.get(
                "oscar_elasticsearch", "query_page_size", 100
            ),
            "OSCAR_ELASTICSEARCH_FACETS": get_facet_config(),
            "OSCAR_ELASTICSEARCH_MIN_NUM_BUCKETS": config.get(
                "oscar_elasticsearch", "min_num_buckets", 2
            ),
            "OSCAR_ELASTICSEARCH_FILTER_AVAILABLE": config.get(
                "oscar_elasticsearch", "filter_available", False
            ),
            "OSCAR_ELASTICSEARCH_PRIORITIZE_AVAILABLE_PRODUCTS": config.get(
                "oscar_elasticsearch", "prioritize_available_products", True
            ),
            "OSCAR_ELASTICSEARCH_DEFAULT_ITEMS_PER_PAGE": DEFAULT_ITEMS_PER_PAGE,
            "OSCAR_ELASTICSEARCH_ITEMS_PER_PAGE_CHOICES": config.get(
                "oscar_elasticsearch",
                "items_per_page_choices",
                [DEFAULT_ITEMS_PER_PAGE],
            ),
            "OSCAR_ELASTICSEARCH_AUTOCOMPLETE_CONTEXTS": [
                {
                    "name": "status",
                    "type": "category",
                    "path": "status",
                }
            ],
            "OSCAR_ELASTICSEARCH_INDEX_UPDATE_SPOOLER_CHUNK_SIZE": 1000,
            "OSCAR_ELASTICSEARCH_INDEX_PREFIX": slugify(PROJECT_NAME),
            "OSCAR_ELASTICSEARCH_FACET_BUCKET_SIZE": config.get(
                "oscar_elasticsearch", "facet_bucket_size", 10
            ),
            "OSCAR_ELASTICSEARCH_HIDE_IMAGELESS_PRODUCTS": config.get(
                "oscar_elasticsearch", "hide_imageless_products", False
            ),
            "OSCAR_ELASTICSEARCH_PRODUCTS_WITH_IMAGES_FIRST": config.get(
                "oscar_elasticsearch", "products_with_images_first", False
            ),
        }

        return merge_dicts(
            settings, OSCAR_ELASTICSEARCH_OVERWRITE_SETTINGS, overwrite=True
        )

    def get_spooler_modules(self, modules):
        modules.append("ocyan.plugin.oscar_elasticsearch.search.spooler")
        return modules

    def get_smoketest_modules(self):
        return ["ocyan.plugin.oscar_elasticsearch.tests"]


class Oscar3ElasticsearchConfig(OscarElasticsearchConfig):
    def contribute_to_installed_apps(self, installed_apps):
        installed_apps.oscar.append(
            "ocyan.plugin.oscar_elasticsearch.search.apps.OscarElasticSearchConfig",
            replaces="oscar.apps.search.apps.SearchConfig",
        )
        installed_apps.oscar.register_extension(
            "oscar_elasticsearch.search", "oscar.apps.search"
        )
        installed_apps.oscar.register_extension(
            "ocyan.plugin.oscar_elasticsearch.search", "oscar_elasticsearch.search"
        )
        installed_apps.oscar.append(
            "ocyan.plugin.oscar_elasticsearch.catalogue_es_extension",
            extends="ocyan.plugin.oscar_catalogue.catalogue",
        )
        installed_apps.oscar.register_extension(
            "ocyan.plugin.oscar_elasticsearch.catalogue_dashboard_es_extension.dashboard.catalogue",
            "ocyan.plugin.oscar_catalogue_dashboard.dashboard.catalogue",
        )
        installed_apps.wagtail.append("wagtail.contrib.search_promotions")
        return installed_apps



from .apps import OscarElasticSearchConfig

# Compatibility export for projects that still reference the legacy app config
# path. Django 5 discovers the explicit AppConfig registered by the entry point.
SearchConfig = OscarElasticSearchConfig



# -*- coding: utf-8 -*-
from django.apps import AppConfig


class OcyanPluginOscarElasticsearchAppConfig(AppConfig):
    name = "ocyan.plugin.oscar_elasticsearch"
    label = "ocyan_plugin_oscar_elasticsearch"



"""
Use the Oscar Elasticsearch backend client for catalogue/search operations.

This keeps query and indexing paths on the same client/index settings.
"""

from oscar_elasticsearch.search.backend import es



from django.dispatch import receiver

from ocyan.core.signals import post_loaddemodata

from .spooler import update_oscar_index as update_oscar_index_async


@receiver(post_loaddemodata)
def update_oscar_index(has_unloaded_fixtures, **kwargs):  # pylint: disable=W0613
    if has_unloaded_fixtures:
        update_oscar_index_async.spool()



from django.conf import settings

from oscar_elasticsearch.search import update as oscar

from ocyan.core.utils import chunked

from . import spooler


class UpdateIndex(oscar.UpdateIndex):
    def synchronize_searchindex(self, **kwargs):
        if self._categories or self._products:
            categories = list(self._categories)
            self._categories = set()
            products = list(self._products)
            self._products = set()

            # We chunk the spooler files because there is a maximum lenght of the spooler files.
            if products:
                for chunk in chunked(
                    products,
                    settings.OSCAR_ELASTICSEARCH_INDEX_UPDATE_SPOOLER_CHUNK_SIZE,
                ):
                    spooler.update_index_products.spool(*chunk)
            if categories:
                for chunk in chunked(
                    categories,
                    settings.OSCAR_ELASTICSEARCH_INDEX_UPDATE_SPOOLER_CHUNK_SIZE,
                ):
                    spooler.update_index_categories.spool(*chunk)



"""The modern Oscar Elasticsearch integration has no Django search models.

``django-oscar-elasticsearch`` 3.x indexes catalogue models through its own
Elasticsearch API. Keeping the old Wagtail Indexed proxy models here would
reintroduce the removed ``wagtail-extendedsearch`` dependency and create
untracked migration state.
"""



from django.conf import settings
from django.db.models import Q

from wagtail.search.utils import separate_filters_from_query

from oscar.apps.dashboard.catalogue import views as oscar
from oscar.core.loading import get_class, get_model

try:
    ProductElasticsearchIndex = get_class(
        "search.api.product", "ProductElasticsearchIndex"
    )
except Exception:
    ProductElasticsearchIndex = None
Product = get_model("catalogue", "Product")
try:
    paginate_result = get_class("search.api.pagination", "paginate_result")
except Exception:

    def paginate_result(rows, _count, _per_page):
        return rows


class SearchResultTableQuerysetSource(object):
    """
    This thing is needed to make django_tables2 think we are dealing with a queryset
    """

    def __init__(self, results, count):
        self._count = count
        self.results = results

    def __len__(self):
        return self._count

    def count(self):
        return self._count

    def exists(self):
        return self._count > 0

    @property
    def model(self):
        return self.results.model

    def order_by(self, *_):
        # there is no ordering possible here
        return self

    def __iter__(self):
        return self.results.iterator()

    def __getitem__(self, key):
        return self.results[key]

    def __call__(self, rows, per_page, *args, **kwargs):
        # this is the paginator
        return paginate_result(rows, self._count, per_page)


class ProductListView(oscar.ProductListView):
    SORT_BY_MAP = {"upc": "code"}

    def get_queryset(self):
        """
        Build the queryset for this list
        """
        queryset = Product.objects.browsable_dashboard().base_queryset()
        queryset = self.filter_queryset(queryset)
        queryset = self.apply_search(queryset)
        return queryset

    def get_table(self, **kwargs):
        if "q" in self.request.GET and not "sort" in self.request.GET:
            kwargs["order_by"] = ""
        return super().get_table(**kwargs)

    def get_table_pagination(self, table):
        table_pagination = super().get_table_pagination(table)
        original_data = table.data.data

        if isinstance(original_data, SearchResultTableQuerysetSource):
            table_pagination["paginator_class"] = original_data

        return table_pagination

    def apply_search(self, queryset):
        """
        Filter the queryset and set the description according to the search
        parameters given
        """
        # pylint: disable=attribute-defined-outside-init
        if "q" in self.request.GET:
            self.form = self.form_class(self.request.GET)
        else:
            self.form = self.form_class()

        if not self.form.is_valid():
            return queryset

        data = self.form.cleaned_data
        context = data["context"]
        q = data["q"]
        if q:
            if context == "all":
                filters, query_string = separate_filters_from_query(q)

                default_filters = [
                    {"terms": {"structure": [Product.PARENT, Product.STANDALONE]}},
                ]

                p = ProductElasticsearchIndex()
                try:
                    page = int(self.request.GET.get("page", 1)) - 1
                except ValueError:
                    page = 0

                from_ = page * settings.OSCAR_DASHBOARD_ITEMS_PER_PAGE

                results, num_found = p.search(
                    query_string=query_string,
                    filters=default_filters + p.format_filters(filters),
                    from_=from_,
                    sort_by=p.format_order_by(self.request.GET.getlist("sort")),
                    to=from_ + settings.OSCAR_DASHBOARD_ITEMS_PER_PAGE,
                )
                queryset = SearchResultTableQuerysetSource(results, num_found)
            elif context == "title":
                queryset = queryset.filter(title__icontains=q.strip())
            elif context == "upc":
                matches_upc = Product.objects.filter(upc=q)
                qs_match = queryset.filter(
                    Q(id__in=matches_upc.values("id"))
                    | Q(id__in=matches_upc.values("parent_id"))
                )

                if qs_match.exists():
                    queryset = qs_match
                else:
                    matches_upc = Product.objects.filter(upc__icontains=q.strip())
                    queryset = queryset.filter(
                        Q(id__in=matches_upc.values("id"))
                        | Q(id__in=matches_upc.values("parent_id"))
                    )

        return queryset



import unittest
from time import sleep

from django.conf import settings
from django.core.management import call_command
from django.test import TestCase
from django.urls import reverse

from oscar.core.loading import get_class, get_model

Product = get_model("catalogue", "Product")
update_index_products = get_class("search.helpers", "update_index_products")


class ElasticSearchViewTest(TestCase):
    fixtures = [
        "search/auth",
        "search/catalogue",
    ]

    @classmethod
    def setUpClass(cls):
        # clear search index
        try:
            call_command("update_oscar_index")
        except Exception as exc:  # pragma: no cover
            raise unittest.SkipTest(
                f"Elasticsearch is unavailable for test setup: {exc}"
            ) from exc
        super().setUpClass()

    def setUp(self):
        super().setUp()
        update_index_products(Product.objects.values_list("id", flat=True))
        sleep(3)

    def test_search_bikini(self):
        url = reverse("search:search")
        response = self.client.get("%s?q=bikini" % url)
        self.assertContains(response, "Hermes Bikini")

    def test_search_suggestion(self):
        url = reverse("search:search")
        response = self.client.get("%s?q=prods" % url)
        self.assertEqual(response.context["suggestion"], "produ")

    def test_opensearchdescription(self):
        url = reverse("search:opensearchdescription")
        response = self.client.get(url)
        self.assertContains(response, "application/opensearchdescription+xml")

    def test_opensearchdescription_search(self):
        url = reverse("search:opensearch-autocomplete")
        response = self.client.get("%s?q=h" % url)
        self.assertEqual(response.json(), ["h", ["Hermes Bikini", "Hubble Photo"]])

    def test_opensearchdescription_crappy_browser(self):
        url = reverse("search:opensearch-autocomplete")
        response = self.client.get("%s?searchTerms={{searchTerms}}" % url)
        self.assertEqual(response.json(), [None, []])

    def test_browse(self):
        url = reverse("catalogue:index")
        response = self.client.get(url)
        self.assertContains(response, "second")
        self.assertContains(response, "serious product")
        self.assertContains(response, "Hubble Photo")
        self.assertContains(response, "Hermes Bikini")

    def test_catagory(self):
        url = reverse("catalogue:category", args=("root", 1))
        response = self.client.get(url)
        self.assertContains(response, "second")
        self.assertContains(response, "serious product")
        self.assertContains(response, "Hubble Photo")
        self.assertContains(response, "Hermes Bikini")

    def test_no_stockrecord_and_not_public(self):
        self.test_catagory()

        hermes = Product.objects.get(pk=2)
        hermes.stockrecords.all().delete()
        hermes.save()

        hubble = Product.objects.get(pk=3)
        hubble.is_public = False
        hubble.save()

        update_index_products([hermes.pk, hubble.pk])
        sleep(2)

        url = reverse("catalogue:category", args=("root", 1))
        response = self.client.get(url)
        self.assertContains(response, "second")
        self.assertContains(response, "serious product")
        self.assertNotContains(response, "Hubble Photo")
        self.assertNotContains(response, "Hermes Bikini")


class OldSettingsTest(TestCase):
    def test_autocomplete_filter(self):
        if hasattr(settings, "OSCAR_ELASTICSEARCH_SUGGESTION_STATUS_FILTER"):
            self.fail(
                "OSCAR_ELASTICSEARCH_SUGGESTION_STATUS_FILTER is outdated, please use OSCAR_ELASTICSEARCH_AUTOCOMPLETE_STATUS_FILTER"
            )



import functools
import logging

from django.http import HttpResponse
from django.utils.translation import gettext as _
from rest_framework import exceptions, status
from rest_framework.authentication import BasicAuthentication

__all__ = ("oscar_auth",)

logger = logging.getLogger(__name__)

FEED_PASSWORD = "912cf9812ffaccc168c3f04722eb4b66060c2c99e467fe51"
FEED_USERNAME = "superuser"


class SimpleBasicAuthentication(BasicAuthentication):
    def authenticate_credentials(self, userid, password, request=None):
        if userid == FEED_USERNAME and password == FEED_PASSWORD:
            return True

        raise exceptions.AuthenticationFailed(_("Invalid username/password."))

    def check(self, request):
        if self.authenticate(request) is None:
            raise exceptions.AuthenticationFailed(_("Missing basic authentication"))


def oscar_auth(func):
    @functools.wraps(func)
    def wrapper(request):
        auth = SimpleBasicAuthentication()
        try:
            auth.check(request)
            return func(request)
        except exceptions.AuthenticationFailed:
            response = HttpResponse(status=status.HTTP_401_UNAUTHORIZED)
            response["WWW-Authenticate"] = auth.authenticate_header(request)
            return response

    return wrapper



import base64

from django.contrib.auth import get_user_model
from django.test import TestCase
from django.urls import reverse

User = get_user_model()


class AuthTest(TestCase):
    fixtures = ["search/auth"]

    auth_headers = {
        "HTTP_AUTHORIZATION": "Basic "
        + base64.b64encode(
            bytes("superuser:912cf9812ffaccc168c3f04722eb4b66060c2c99e467fe51", "utf8")
        ).decode("utf8")
    }

    def test_auth_initialized(self):
        self.assertEqual(User.objects.count(), 2)

    def test_insufficient_rights(self):
        henk = User.objects.get(username="regularuser")
        self.client.force_login(henk)

        response = self.client.get(
            reverse("search:update_oscar_index"), **self.auth_headers
        )

        self.assertEqual(response.status_code, 302)
        self.assertEqual(
            response.url, "/manage/login/?next=/search/update_oscar_index/"
        )

    def test_sufficient_rights(self):
        henk = User.objects.get(username="admin")
        self.client.force_login(henk)

        response = self.client.get(
            reverse("search:update_oscar_index"), **self.auth_headers
        )

        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, "/admin")

    def test_wrong_password(self):
        henk = User.objects.get(username="admin")
        self.client.force_login(henk)

        auth_headers = {
            "HTTP_AUTHORIZATION": "Basic "
            + base64.b64encode(bytes("superuser:verkeerd_password", "utf8")).decode(
                "utf8"
            )
        }

        response = self.client.get(reverse("search:update_oscar_index"), **auth_headers)

        self.assertEqual(response.status_code, 401)



"""Regression checks for the supported Wagtail 7/Oscar search contract."""

from pathlib import Path
import unittest

PACKAGE_ROOT = (
    Path(__file__).resolve().parents[1] / "ocyan" / "plugin" / "oscar_elasticsearch"
)


class ModernSearchContractTests(unittest.TestCase):
    def test_removed_wagtail_search_contract_is_not_reintroduced(self):
        source = "\n".join(
            path.read_text()
            for path in PACKAGE_ROOT.rglob("*.py")
            if "tests" not in path.parts
        )

        self.assertNotRegex(source, r"(?:from|import)\s+wagtail_extendedsearch")
        self.assertNotRegex(
            source,
            r"from\s+wagtail\.search\.backends\.elasticsearch6\s+import",
        )

    def test_oscar_elasticsearch_three_is_the_runtime_contract(self):
        setup_source = (PACKAGE_ROOT.parents[2] / "setup.py").read_text()
        views_source = (PACKAGE_ROOT / "search" / "views.py").read_text()

        self.assertIn('"django-oscar-elasticsearch>=3,<4"', setup_source)
        self.assertIn('"elasticsearch>=8,<9"', setup_source)
        self.assertIn('"search.views.search"', views_source)
        self.assertIn('"CatalogueAutoCompleteView"', views_source)



from oscar_elasticsearch.search import apps


class OscarElasticSearchConfig(apps.OscarElasticSearchConfig):
    name = "ocyan.plugin.oscar_elasticsearch.search"

    def ready(self):
        super().ready()
        # pylint: disable=unused-import
        from . import search_signal_handlers, signal_handlers

    def get_urls(self):
        from .urls import urlpatterns

        return super().get_urls() + urlpatterns



from django.http import JsonResponse
from django.contrib.admin.views.decorators import staff_member_required
from django.shortcuts import redirect

from oscar.core.loading import get_class

from .auth import oscar_auth
from .spooler import update_oscar_index as update_index_async

BaseCatalogueAutoCompleteView = get_class(
    "search.views.search", "CatalogueAutoCompleteView"
)


@oscar_auth
@staff_member_required
def update_oscar_index(request):
    update_index_async.spool()
    return redirect("/admin")


class OpenSearchAutocompleteView(BaseCatalogueAutoCompleteView):
    def get(self, request, *args, **kwargs):
        results = self.get_suggestions()
        return JsonResponse([self.request.GET.get("q"), results], safe=False)



import odin

from oscar.core.loading import get_class, get_model

from ocyan.main.loading import get_baseclass

ProductResource = get_class("oscar_odin.resources.catalogue", "ProductResource")

ProductElasticSearchResource = get_class(
    "search.mappings.products.resources", "ProductElasticSearchResource"
)

BaseProductMapping = get_baseclass(
    "search.mappings.products.mappings",
    "ProductMapping",
    "ocyan.plugin.oscar_elasticsearch.search",
)
Product = get_model("catalogue", "Product")


class ProductMapping(BaseProductMapping):
    from_resource = ProductResource
    to_resource = ProductElasticSearchResource

    register_mapping = False

    @odin.map_field(from_field="is_public")
    def is_public(self, is_public):
        if self.source.structure == Product.PARENT:
            has_stockrecords = (
                Product.objects.filter(parent_id=self.source.id)
                .filter(stockrecords__isnull=False)
                .exists()
            )
        else:
            has_stockrecords = (
                False if not self.source.price and not self.source.currency else True
            )

        if not has_stockrecords:
            return False

        return is_public



from django.conf import settings
from django.utils.translation import get_language

from oscar_elasticsearch.search.indexing.settings import *  # noqa: F403,F401
from oscar_elasticsearch.search.settings import INDEX_PREFIX


def _lang_suffix() -> str:
    # Catalogue indices in existing projects are commonly generated with `-en`.
    # Allow explicit override, fallback to active language, then `en`.
    lang = (
        getattr(settings, "OSCAR_ELASTICSEARCH_CATALOGUE_LANGUAGE", None)
        or get_language()
        or "en"
    ).lower()
    return lang.split("-")[0]


# Keep storefront/search queries aligned with language-aware indices.
OSCAR_PRODUCTS_INDEX_NAME = f"{INDEX_PREFIX}__catalogue_product-{_lang_suffix()}"
OSCAR_CATEGORIES_INDEX_NAME = f"{INDEX_PREFIX}__catalogue_category-{_lang_suffix()}"



from ocyan.main.loading import get_baseclasses

# pylint: disable=unbalanced-tuple-unpacking
BaseCatalogueView, BaseProductCategoryView = get_baseclasses(
    "catalogue.views",
    ["CatalogueView", "ProductCategoryView"],
    "ocyan.plugin.oscar_elasticsearch.catalogue_es_extension",
)


class CatalogueView(BaseCatalogueView):
    order_by_relevance = False


class ProductCategoryView(BaseProductCategoryView):
    order_by_relevance = False



from django.urls import path
from django.views.generic import TemplateView

from .views import OpenSearchAutocompleteView, update_oscar_index

urlpatterns = [
    path(
        "opensearchdescription.xml",
        TemplateView.as_view(
            template_name="oscar/search/opensearchdescription.xml",
            content_type="application/opensearchdescription+xml",
        ),
        name="opensearchdescription",
    ),
    path(
        "opensearch-autocomplete/",
        OpenSearchAutocompleteView.as_view(),
        name="opensearch-autocomplete",
    ),
    path("update_oscar_index/", update_oscar_index, name="update_oscar_index"),
]



"""
Command to cleanup unused elasticsearch indices
"""

from django.conf import settings
from django.core.management.base import BaseCommand
from django.utils.translation import activate

from wagtail.search.backends import get_search_backend
from wagtail.search.index import get_indexed_models
from wagtail.search.management.commands.update_index import group_models_by_index

from ocyan.core.fender import config


class Command(BaseCommand):
    help = __doc__

    def add_arguments(self, parser):
        parser.add_argument(
            "--remove-stale", action="store_true", help="Remove all stale indices"
        )

    def handle(self, *args, **options):
        self.stdout.write("cleaning up old elasticsearch indices")

        if config.i18n_enabled:
            for lang, _ in settings.LANGUAGES:
                activate(lang)
                self.cleanup_indices(options)
        else:
            self.cleanup_indices(options)

    def cleanup_indices(self, options):
        backend = get_search_backend("default")

        models_grouped_by_index = group_models_by_index(
            backend, get_indexed_models()
        ).items()
        if not models_grouped_by_index:
            self.stdout.write("No indices found")
            return

        stale_indices = []

        for es_index, _ in models_grouped_by_index:
            (alias,) = backend.es.cat.aliases(es_index.name, format="json")
            index_name = alias["index"]
            for index in backend.es.cat.indices("%s_*" % es_index.name, format="json"):
                if index["index"] == index_name:
                    msg = (
                        "Active index %(index)s is %(health)s %(docs.count)s %(store.size)s"
                        % index
                    )
                    if index["health"] == "green":
                        out_msg = self.style.SUCCESS(msg)
                    else:
                        out_msg = self.style.WARNING(msg)
                    self.stdout.write(out_msg)
                else:
                    stale_indices.append(index)

        if stale_indices:
            self.stdout.write(self.style.WARNING("Stale indices found"))
            for stale_index in stale_indices:
                self.stdout.write(
                    " - stale index %(index)s %(health)s %(docs.count)s %(store.size)s"
                    % stale_index
                )

            if options["remove_stale"]:
                self.stdout.write(self.style.ERROR("REMOVING STALE INDICES"))
                for stale_index in stale_indices:
                    self.stdout.write(
                        self.style.ERROR(
                            "REMOVING %(index)s is %(health)s %(docs.count)s %(store.size)s"
                            % stale_index
                        )
                    )
                    backend.es.indices.delete(stale_index["index"])
            else:
                self.stdout.write(self.style.SUCCESS("Not removing anything"))
        else:
            self.stdout.write(self.style.SUCCESS("No stale indices found"))
